import threading
import subprocess
import json
import logging
import os

logger = logging.getLogger('daemon.' + __name__)

class RemoteAccessHandler:
    def __init__(self, device_client_instance):
        self.device_client = device_client_instance
        self.tunnel = None
        self.host="myurserver"
        self.home = os.path.expanduser('~')
        self.ssh_config_file = os.path.join(self.home, ".ssh/ur_config")
        self.ssh_identity_file = os.path.join(self.home, ".ssh/ur_id_ed25519")
        self.ssh_known_hosts_file = os.path.join(self.home, ".ssh/known_hosts")


    def create_ssh_tunnel(self, correlation_id, domain):
        logger.debug("Creating ssh tunnel")
        self.tunnel = threading.Thread(target=self._create_ssh_tunnel, args=(correlation_id, domain))
        self.tunnel.daemon = True
        self.tunnel.start()

    def _create_ssh_tunnel(self, correlation_id, domain):
        host_name = domain.split("@")[1]
        try :
            conf = dict (
                HostName = str(host_name),
                User = str(domain.split("@")[0]),
                IdentityFile = self.ssh_identity_file,
                ServerAliveInterval = "60",
                ServerAliveCountMax = "3",
                ExitOnForwardFailure = "yes",
                ControlMaster = "auto",
                ControlPath = "/tmp/ssh_tunnel_socket",
                ControlPersist = "2m",
                StrictHostKeyChecking = "no"
            )

            with open(self.ssh_config_file, "w") as f:
                f.write("Host {0}\n".format(self.host))
                for key, value in conf.items():
                    f.write("\t%s %s\n".encode("utf-8") % (key, value))

            cmd_array = ["ssh", "-p 443", "-F", self.ssh_config_file, "-fNTCR", "0:localhost:6080", self.host]
            logger.debug("Creating ssh tunnel with command : " + str(cmd_array))
            subprocess.check_call(cmd_array)
        except subprocess.CalledProcessError as e:
            logger.error(str(e))
            raise e

        message = json.dumps({
            "type": "VncAccess",
            "messageType": "VncTunnelCreated",
            "correlationId": str(correlation_id)
        })
        self._send_message_to_hub(message)


    def send_remote_request_access_response(self, correlation_id, accepted):
        logger.debug("send_remote_request_access_response for request id : " + str(correlation_id))
        if accepted == True:
            try:
                subprocess.call("ssh-keygen -t ed25519 -f /root/.ssh/ur_id_ed25519 -q -N '' <<< y", shell=True)
                with open("/root/.ssh/ur_id_ed25519.pub","r") as f:
                    publicKey = f.read()
                message = json.dumps({
                    "type": "VncAccess",
                    "messageType": "VncRequestUserResponse",
                    "AccessRequestAccepted": "True",
                    "correlationId": str(correlation_id),
                    "PublicKey": str(publicKey)
                })
                self._send_message_to_hub(message)
                return True
            except subprocess.CalledProcessError as e:
                logger.error(str(e))

        # If the access request is not accepted
        message = json.dumps({
            "type": "VncAccess",
            "messageType": "VncRequestUserResponse",
            "AccessRequestAccepted": "False",
            "correlationId": str(correlation_id)
        })
        self._send_message_to_hub(message)


    def send_remote_access_client_connected_event(self, correlation_id):
        message = json.dumps({
            "type": "VncAccess",
            "messageType": "VncClientConnected",
            "correlationId": str(correlation_id)
        })
        self._send_message_to_hub(message)

    def close_ssh_tunnel(self, correlation_id):
        message = json.dumps({
            "type": "VncAccess",
            "messageType": "VncTunnelClosed",
            "correlationId": str(correlation_id)
        })
        self._send_message_to_hub(message)
        try:
            cmd_array = ["ssh", "-F", self.ssh_config_file, "-O", "exit", self.host]
            subprocess.check_call(cmd_array)
        except subprocess.CalledProcessError as e:
            logger.error(str(e))
        except Exception as e:
            logger.error(str(e))
        try :
            if os.path.exists(self.ssh_identity_file):
                os.remove(self.ssh_identity_file)
                os.remove(self.ssh_identity_file + ".pub")
            if os.path.exists(self.ssh_config_file):
                os.remove(self.ssh_config_file)
        except Exception as e:
            logger.error("Error in deleting files : " + str(e))

    def _send_message_to_hub(self, message):
        try:
            self.device_client.send_message(message)
            return True
        except Exception as e:
            logger.error("Failed to send message : " + message + " to hub :" + str(e))
            return False

